/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 1991-2005 OpenCFD Ltd.
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM; if not, write to the Free Software Foundation,
    Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

\*---------------------------------------------------------------------------*/

#include "totalMassFluxFvPatchVectorField.H"
#include "addToRunTimeSelectionTable.H"
#include "volFields.H"
#include "surfaceFields.H"
#include "fvcMeshPhi.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

totalMassFluxFvPatchVectorField::totalMassFluxFvPatchVectorField
(
    const fvPatch& p,
    const vectorField& iF
)
:
  fixedValueFvPatchVectorField(p, iF),
  totalMf_(0)
{
}


totalMassFluxFvPatchVectorField::totalMassFluxFvPatchVectorField
(
    const totalMassFluxFvPatchVectorField& ptf,
    const fvPatch& p,
    const vectorField& iF,
    const fvPatchFieldMapper& mapper
)
:
  fixedValueFvPatchVectorField(ptf, p, iF, mapper),
  totalMf_(ptf.totalMf_)
{  
}


totalMassFluxFvPatchVectorField::totalMassFluxFvPatchVectorField
(
    const fvPatch& p,
    const vectorField& iF,
    const dictionary& dict
)
:
  fixedValueFvPatchVectorField(p, iF),
  totalMf_(readScalar(dict.lookup("totalMassFluxPerSecond")))
{
    fvPatchVectorField::operator=(vectorField("value", dict, p.size()));
}


totalMassFluxFvPatchVectorField::totalMassFluxFvPatchVectorField
(
    const totalMassFluxFvPatchVectorField& pivpvf,
    const vectorField& iF
)
:
  fixedValueFvPatchVectorField(pivpvf, iF),
  totalMf_(pivpvf.totalMf_)
{
}


// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //


void totalMassFluxFvPatchVectorField::autoMap
(
    const fvPatchFieldMapper& m
)
{
    vectorField::autoMap(m);
}


void totalMassFluxFvPatchVectorField::rmap
(
    const fvPatchVectorField& ptf,
    const labelList& addr
)
{
    fixedValueFvPatchVectorField::rmap(ptf, addr);
}

// Update the coefficients associated with the patch field
void totalMassFluxFvPatchVectorField::updateCoeffs()
{
    if (updated())
    {
        return;
    }

    const volScalarField& rho = db().lookupObject<volScalarField>("rho");
 
    const fvPatchField<scalar>& rhop =
        patchField<volScalarField, scalar>(rho);

    const vectorField& n = patch().nf();

    const scalar totalArea=sum(patch().magSf());

    operator==(-(n*((totalMf_/totalArea)/rhop)));

    fixedValueFvPatchVectorField::updateCoeffs();

//     Info << *this;
}


// Write
void totalMassFluxFvPatchVectorField::write(Ostream& os) const
{
    fvPatchVectorField::write(os);
    writeEntry("value", os);
    os.writeKeyword("totalMassFluxPerSecond")
        << totalMf_ << token::END_STATEMENT << nl;
}


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

makePatchTypeField(fvPatchVectorField, totalMassFluxFvPatchVectorField);

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace Foam

// ************************************************************************* //
